// Decompiled by Jad v1.5.8g. Copyright 2001 Pavel Kouznetsov.
// Jad home page: http://www.kpdus.com/jad.html
// Decompiler options: packimports(3)
// Source File Name:   TileSlicer.java

import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;
import javax.imageio.ImageIO;

public class TileSlicer
{
    private HashMap<Integer,HashMap<Integer,BufferedImage>> tiles;
    private int tileMap[][];
    private static final String DEFAULT_FILE = "map.png";
    private BufferedImage map;
    private int tileWidth;
    private int tileHeight;

    public static void main(String args[])
        throws IOException
    {
    	BufferedImage map;
        int tileWidth;
        int tileHeight;

        if(args.length > 0)
            map = ImageIO.read(new File(args[0]));
        else
            map = ImageIO.read(new File(DEFAULT_FILE));
        
        if(args.length > 1)
            tileWidth = Integer.parseInt(args[1]);
        else
            tileWidth = 16;

        if(args.length > 2)
            tileHeight = Integer.parseInt(args[2]);
        else
            tileHeight = tileWidth;
        
        try
        {
        	TileSlicer slicer = new TileSlicer(map,tileWidth,tileHeight);
            slicer.slice();
            slicer.writeToFiles();
        }
        catch (Exception e)
        {
        	System.out.println(e.getMessage());
        }
    }

    public TileSlicer(BufferedImage map,int tileWidth, int tileHeight) throws Exception
    {
    	this.map = map;
    	this.tileWidth = tileWidth;
    	this.tileHeight = tileHeight;

    	tiles = new HashMap<Integer,HashMap<Integer,BufferedImage>>();
        tileMap = new int[map.getWidth() / tileWidth][map.getHeight() / tileHeight];
        if(map.getWidth() % tileWidth != 0 || map.getHeight() % tileHeight != 0)
        {
            throw new Exception("ERROR: Image dimensions not a multiple of the tile size");
        }
    }

    private void slice()
    {
        System.out.println("Slicing...");
        int index = 0;
        int count = 0;
        int max = (map.getHeight() / tileHeight) * (map.getWidth() / tileWidth);
        for(int y = 0; y < map.getHeight(); y += tileHeight)
        {
            for(int x = 0; x < map.getWidth(); x += tileWidth)
            {
                System.out.print((float)(count++)/max*100 + "%          \r");
                BufferedImage tempTile = map.getSubimage(x, y, tileWidth, tileHeight);
                int hash = getTileHash(tempTile);

                HashMap<Integer,BufferedImage> imgs = tiles.get(hash);
                if (imgs == null)
                {
                    // assign new set and add tile
                    tileMap[x / tileWidth][y / tileHeight] = index;

                    imgs = new HashMap<Integer,BufferedImage>();
                    imgs.put(index++,tempTile);

                    tiles.put(hash,imgs);
                }
                else
                {
                    // see if matching image is in list
                    boolean found = false;
                    Iterator<Map.Entry<Integer,BufferedImage>> iter = imgs.entrySet().iterator();
                    while (!found && iter.hasNext())
                    {
                        Map.Entry<Integer,BufferedImage> entry = iter.next();

                        if (tilesAreEqual(tempTile,entry.getValue()))
                        {
                            // match
                            found = true;
                            tileMap[x / tileWidth][y / tileHeight] = entry.getKey();
                        }
                    }
                    // check for no match
                    if (!found)
                    {
                        // add
                        tileMap[x / tileWidth][y / tileHeight] = index;
                        imgs.put(index++, tempTile);
                    }
                }
            }
        }
        System.out.println("Done - " + index + " unique tiles found.");
    }

    private void writeToFiles()
        throws IOException
    {
        System.out.println("Writing to disk...");
        int count = 0;
        Iterator<HashMap<Integer,BufferedImage>> tileIt = tiles.values().iterator();
        while (tileIt.hasNext())
        {
            HashMap<Integer,BufferedImage> imgs = tileIt.next();
            Iterator<Map.Entry<Integer,BufferedImage>> iter = imgs.entrySet().iterator();
            while (iter.hasNext())
            {
                Map.Entry<Integer,BufferedImage> entry = iter.next();
                ImageIO.write((BufferedImage)entry.getValue(), "png", new File(entry.getKey() + ".png"));
                count++;
            }
        }

        FileWriter out = new FileWriter("map.txt");
        FileWriter html = new FileWriter("map.html");
        out.write("Dimensions: " + tileMap.length + " x " + tileMap[0].length + "\nUnique tiles: " + count + "\n\nMap Data:\n");
        html.write("<html><head><title>Map</title></head>\n<body>Dimensions: " + tileMap.length + "&times;" + tileMap[0].length + "<br>Unique tiles: " + count + "<br><br>Map Data:<br><nobr>\n");
        for(int y = 0; y < tileMap[0].length; y++)
        {
            for(int x = 0; x < tileMap.length; x++)
            {
                out.write(tileMap[x][y] + " ");
                html.write("<img src=\""+tileMap[x][y] + ".png\">");
            }

            out.write("\n");
            html.write("<br>\n");
        }

        html.write("</nobr></body></html>");
        out.close();
        html.close();
    }

    private boolean tilesAreEqual(BufferedImage a, BufferedImage b)
    {
        for(int y = 0; y < tileHeight; y++)
            for(int x = 0; x < tileWidth; x++)
                if(a.getRGB(x, y) != b.getRGB(x, y))
                    return false;
        return true;
    }

    private int getTileHash(BufferedImage tile)
    {
        int result = 0;
        for(int y = 0; y < tileHeight; y++)
            for(int x = 0; x < tileWidth; x++)
                result += tile.getRGB(x, y);
        return result;
    }
}
